{% load compress static %}{% spaceless %}

{% compress js %}
    <script>alert('amazing');</script>
    <script type="text/javascript" src="{% static "js/one.js" %}"></script>
{% endcompress %}{% endspaceless %}
