{% spaceless %}
{% block js %}
    <script type="text/javascript">
        alert("test using template extension passed in variable parent=base1.html");
    </script>
{% endblock %}
{% endspaceless %}
