{% spaceless %}
{% block js %}
    <script type="text/javascript">
        alert("test using block.super");
    </script>
{% endblock %}

{% block css %}
    <style type="text/css">
        body {
            background: red;
        }
    </style>
{% endblock %}
{% endspaceless %}
